/*
Copyright (C) 2004-2005 Amour Hassan

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef ARENA_H
#define ARENA_H

#include <rgOpenGL.h>
#include "../things/things.h"


class phArena : public gmModelThing
{
public:
	phArena();
	~phArena();

	void clear();
	int load( const char *fname);
	void draw_ice_surface() const {
		if( dsplst_ice_surface) glCallList( dsplst_ice_surface);}

	void process( ThingManager *tm);
	void draw_boards() const;
	void draw_glass() const;

	void clear_display_list( GLuint &id);

	std::string type() const { return std::string( "phArena");}
	bool is_a( const std::string &s) const {
		if( s != phArena::type()) return gmModelThing::is_a( s);
		return true;
	}

	int call_ccmd( const std::string &key, gmCCArg &args);
	std::string list_ccmd( const std::string &p_prefix);

	bool add_opaque( ThingControlBlock *tcb);
	void remove_opaque( ThingControlBlock *tcb);

protected:
	rgVector m_ice_color;
	rgVector m_light_level;
	float m_ice_alpha;
	bool m_reflections;

	// The arena class keeps a number of render lists because certain
	// things require special rendering techniques. The rendering of the
	// arena, players and all other objects is tightly coupled do to the
	// lack of an acceptable generic rendering algorithm.
	std::map <ThingControlBlock *, void *> m_opaque;

	static std::map <const std::string, int (*)( const gmCCArg &)> m_ccmd;

	GLuint dsplst_ice_surface;
	GLuint dsplst_boards;

	void compile_face_list( const char *p_name, GLuint &id);
	void compile_face_list( const char *p_name[], const int p_count,
		GLuint &id);

	void render_all_opaque( gmRenderArgs *);

	friend int phArena_ice_color( const gmCCArg &p);
	friend int phArena_ice_alpha( const gmCCArg &p);
	friend int phArena_light_level( const gmCCArg &p);
	friend int phArena_reflections( const gmCCArg &p);
	friend int phArena_prepare_for_rendering( const gmCCArg &p);
	friend int phArena_add_opaque( const gmCCArg &p);
	friend int phArena_remove_opaque( const gmCCArg &p);

private:
	phArena( const phArena &);
	phArena &operator =( const phArena &);
};

#endif
